<?php
// PHP-Programm zur Anzeige der Herrnhuter Losungen in einer Internetseite.
// Fassung vom 6.August 2017
//    - für PHP 5.2.3 und später , mit Einschränkung ab PHP 4.3
// Autor: W. Pavel, E-Post: post@wpavel.de  |  Hausseite: www.wpavel.de
// Für die Apostelkirche Gerbrunn:
//    E-Post: pavel@apostelkirche-gerbrunn.de | www.apostelkirche-gerbrunn.de
// Zur Verwendung der CSV-Datei der Evangelischen Brüder-Unität –
// Herrnhuter Brüdergemeine (http://www.ebu.de)
// erhältlich unter http://www.losungen.de
// Beim Einsatz des Skripts beachte man die Nutzungsbestimmungen
// für die CSV-Datei.
/////////////////////////////////////////////////////////////////////////////
// Das Skript verarbeitet CSV-Dateien in den Kodierungen
// UTF-16LE (2015), UTF-16BE und UTF-8, Apple-Macintosh (2016), Windows-1252
// sowie ISO-8859-1 (bis 2014). soweit mit Windows-1252 identisch 
// Ausgabe normalerweise in UTF-8 , optional in ISO-8859-1
/////////////////////////////////////////////////////////////////////////////

function deflos_fehler($m) {
   // nur für interne Zwecke dieses Programms
   global $DEFLOS;
   $DEFLOS=$m;
   return false; }

function deflos_sp($z,$s) {
	// nur für interne Zwecke dieses Programms
	global $SP;
	if(!array_key_exists($s,$SP)) return false;
	return $z[$SP[$s]]; }   

function deflos($d="",$tp="",$iso=0) {
   // muss im HTML-Dokument mindestens einmal aufgerufen werden.
   // 1. Parameter: die CSV-Datei mit den Losungen,
   //    ggf. mit relativem Pfad zur HTML-Datei;
   //    voreingestellt bei Nichtangabe oder Angabe "": "losungen.csv";
   // 2.Parameter: Datum der gewünschten Losung
   //  	als string in der deutschen  Schreibweise, also z.B. "3.11.2007",
   //		oder als timestamp (Integer) wie durch time() geliefert; 
   //    kann für das aktuelle Jahr abgekürzt werden: "3.11." oder "3.11".
   //    Voreingestellt ist, wenn nicht angegeben oder "": aktuelles Datum (lt. Server)
   //    Darf auch 1 oder 2 (integer) sein, wenn kein 3. Parameter angegeben ist;
   //			wird dann als 3. Parameter behandelt; für Datum Voreinstellung.	
   // 3.Parameter: Ausgabe-Kodierung: 
   //   0=UTF-8 (voreingestellt); 1=ISO-8859-1; 2=ISO-8859-1 mit HTMLEntities
   //   kann in den Funktionen AT,NT,...,AT_,NT_, abweichend angegeben werden
   // Rückgabewert: assoziatives Array mit den 7 Elementen:
   //   array["AT"], array["AV"], array["NT"], array["NV"], array["LD"] , array["ISO"]:
   //   jedes Element davon entspricht einem Text;
   //   das letzte (ISO) enthält die Einstellung entsprechend 3. Parameter. 
   // Im globalen Array $LOSUNG steht das Ergebnis des (letzten) Aufrufs
   //    von deflos zur Verfügung, kann aber auch einer beliebigen Variablen
   //    zugewiesen werden.
   // Bei groben Fehlern liefert deflos das Ergebnis false,
   //   das Ergebnisarray, auch $LOSUNG, ist dann auch false.
   // In der Variablen $DEFLOS steht der Fehlertext oder ok.
   // Bei Fehlern in Teilen einer sonst richtigen Losung ist das
   // entsprechende der 5 Arrayelemente gleich "" .
   global $LOSUNG; // Standard-Array für die gewünschten Losungsteile
   global $ISO; // Wert von Parameter $iso (siehe oben)
   global $AT,$AV,$NT,$NV,$LD; // die Ergebnis-Texte
   global $SP; // Spaltenüberschriften der CSV-Datei
   global $DEFLOS; // OK oder Fehlertext
   $wt=array("Sonntag","Montag","Dienstag","Mittwoch",
   			 "Donnerstag","Freitag","Samstag");
   $DEFLOS="Unerwarteter Fehler";
   $tj=time(); $dj=date("Y",$tj);
   if($tp) { // Datum angegeben
 		if(is_int($tp)) { // timestamp oder 1 oder 2 statt 3. Parameter
 			if(func_num_args()<3&&$tp<3) $iso=$tp; else { $tj=$tp; $dj=date("Y",$tj); }}
 			// für Aufruf deflos("...",iso), also Auslassen des 2. Parameters, (iso=tp)<>0
 		else { // Datum als string tag.monat.jahr oder tag.monat  
      	$e=explode(".",$tp);
      	if(count($e)!=2&&count($e)!=3) return
         	deflos_fehler("'".$tp."' ist keine korrekte Datumsangabe");
      	if (count($e)==3) if($e[2]) $dj=trim($e[2]); // mit Jahresangabe
      	if(!checkdate($e[1],$e[0],$dj)) return
         	deflos_fehler("'".$tp."' ist keine korrekte Datumsangabe");
      	$tj=mktime(0,0,0,$e[1],$e[0],$dj); }}
   $h=getdate($tj); $dt=$h["mday"]; $dm=$h["mon"]; $dw=$h["wday"];
   $ISO=$iso; // 0, 1 oder 2
   if(!$d) { // für den möglichen Aufruf deflos("",...)
      if(file_exists("losungen".$dj.".csv")) $d="losungen".$dj.".csv";
         elseif(file_exists("losungen.csv")) $d="losungen.csv";
            else return
               deflos_fehler("(Standard-)Losungsdatei nicht gefunden"); }
   $dp=pathinfo(trim($d));
   if(!$dp["extension"]) {
      $dp["extension"]="csv";
      $dp["basename"]=$dp["basename"].".".$dp["extension"]; }
   if(strtolower($dp["extension"])!="csv")
         return deflos_fehler("'".$d."' hat nicht die Endung CSV");
   $n=$dp["basename"]; // Dateiname
   $p=strrpos($n,"#");
   if(is_integer($p)) { // Jahresersetzung in Dateiname
      $q=strlen($dj); if($p+1<$q) $q=$p+1; // Anzahl möglicher Ziffern
      for($c=0;$c<$q;$c++) {
         if($n[$p-$c]<>"#") break;
         $n[$p-$c]=$dj[strlen($dj)-$c-1]; } // Jahreszahl in Dateinamen
      $dp["basename"]=$n; }
	if(($f=file_get_contents($dp["dirname"]."/".$dp["basename"]))===false)
      return deflos_fehler("Datei '".$d."' nicht gefunden");
	// Zeilenenden bestimmen, evtl. normieren
	$f=str_replace("\n\r","\n",$f); $f=str_replace("\r\n","\n",$f);
	$pn=substr_count($f,"\n"); $pr=substr_count($f,"\r"); 
	if($pn>$pr) $ze="\n"; else $ze="\r";
	// Codierung bestimmen und auf UTF-8 normieren	
   $p=true; // false wenn nicht Datei nicht UTF
	if(substr($f,0,3)==chr(239).chr(187).chr(191)) $cc=""; // UTF-8
		elseif(substr($f,0,2)==chr(255).chr(254)) $cc="UTF-16LE";   
		elseif(substr($f,0,2)==chr(254).chr(255)) $cc="UTF-16BE"; 
		else { 
			$p=false;
			if($ze=="\r") { $f=iconv("macintosh","UTF-8",$f); $cc=""; }	// Mac speziell	
			   else $cc="Windows-1252"; }
	if($cc) $f=mb_convert_encoding($f,"UTF-8",$cc); // Datei komplett in UTF-8 kodieren
	$so=mb_split($ze,$f); // CSV-Zeilen 
  	if($p) $y=substr($so[0],3); else $y=$so[0];
   if(strpos($y,"\t")!==false) $tb=true; else $tb=false;
      // Trenner der CSV-Datei entweder Tab oder Semikolon
	$z=($tb?str_getcsv($y,"\t"):str_getcsv($y,";",'"'));
		// 1. Zeile der CSV-Datei: enthält die Spaltenüberschriften
		// hier verwendet: Datum, Sonntag, Losungstext, Losungsvers, Lehrtext, Lehrtextvers
	$SP=array_flip($z); // Zuordnung Spaltenname zu Index
	if(!array_key_exists("Datum",$SP))
		return deflos_fehler("CSV-Datei enthält keine Spalte 'Datum'");
   $sd=$SP["Datum"]; $x=false; 
   for($s=1;$s<count($so);$s++) { // Losungsdatei zeilenweise abarbeiten
		$z=($tb?str_getcsv($so[$s],"\t"):str_getcsv($so[$s],";",'"')); // Felder der Zeile in Array z
// Patch für 2016: ggf. "Sonntag" einfügen
		if(count($z)<count($SP)) {
			for($j=count($z);$j>2;$j--) $z[$j]=$z[$j-1];
			$z[2]=""; } 		
// Ende Patch		
		$e=explode(".",$z[$sd]); // Datumfeld zerlegen
      if(count($e)!=3) continue; // Titelzeile oder ungültiges Datumfeld
      if($e[0]==$dt&&$e[1]==$dm&&$e[2]==$dj) { $x=true; break; }}
         // Losung zum Tag gefunden
   if(!$x)
      return deflos_fehler(
      "Losung zum ".$dt.".".$dm.".".$dj." in Datei '".$d."' nicht gefunden");
   if(!($LD=deflos_sp($z,"Sonntag"))) $LD=$wt[$dw].", ".$dt.".".$dm.".".$dj; // ggf. spez. Bezeichnung des Tages
   $AT=deflos_sp($z,"Losungstext"); $AV=deflos_sp($z,"Losungsvers");
   $NT=deflos_sp($z,"Lehrtext"); $NV=deflos_sp($z,"Lehrtextvers"); // Losungstexte
   foreach(array("AT","AV","NT","NV","LD","ISO") as $e) $LOSUNG[$e]=${$e}; // Elemente ins Ergebnisarray
   $DEFLOS="OK";
   return $LOSUNG; } // zurück mit Ergebnis

function deflos_tg($t,$x,$ya,$ye) {
   // #...# -> <b>...</b> bzw. /.../ -> <i>...</i>  (x->y)
   $a=true; $p=0;
   while(($p=strpos($t,$x,$p))!==false) {
      $t=substr_replace($t,($e=($a?$ya:$ye)),$p,1);
      $p=$p+strlen($e); $a=!$a; }
   if(!$a) $t=$t.$ye;
   return $t; }

function deflos_tl($k,$a="") {
   // nur für interne Zwecke dieses Programms
   // Erstelle komplette Losungsteil mit Sonderzeichenumwandlung als Text
   // 1.Parameter: Kennung des Teils: AT,AV,NT,NV oder LD
   // 2.Parameter: Name des Arrays, in dem die Losung liegt;
   //    Voreinstellung "" , ergibt $LOSUNG
   if(!$a) $a="LOSUNG";
   if(is_string($a)) { global ${$a}; $lg=${$a}; } else $lg=$a;
   if(!$lg) return false; // vorher Fehler beim Finden der Losung
   if(!is_array($lg)) return false; // falsche Losungsvariable
   if(!$lg[$k]) return false; // vorher Fehler beim Losungsteil
   $t=$lg[$k]; $i=$lg["ISO"];
   if($i) $t=mb_convert_encoding($t,"ISO-8859-1","UTF-8");
   if($i>1) $t=htmlentities($t,ENT_COMPAT,"ISO-8859-1");
   $t=deflos_tg($t,"/","<i>","</i>");
   $t=deflos_tg($t,"#","<b>","</b>");
   return $t; }

function deflos_aus($k,$a="") {
   // nur für interne Zwecke dieses Programms
   // Ausgabe eines kompletten Losungsteils ins HTML-Dokument
   // Parameter wie in deflos_tl
   if($t=deflos_tl($k,$a)) { echo $t; return true; }
   else return false; }

// Funktionen zum Abrufen und Ausgeben der Losungsteile aus dem HTML-Dokument
// 1.Parameter: Name des Losungsarrays, voreingestellt "" ( ergibt Standardarray $LOSUNG)
// Ergebnis von AT_, AV_, NT_, NV_, LD_ :
//    der Losungsteil als Text; keine Ausgabe
// Ergebnis von AT, AV, NT, NV, LD :
//    true; Ausgabe des Losungsteils ins HTML-Dokument
// bei Fehler in beiden Fällen Ergebnis false

function AT($a="") {return deflos_aus("AT",$a);}
function AV($a="") {return deflos_aus("AV",$a);}
function NT($a="") {return deflos_aus("NT",$a);}
function NV($a="") {return deflos_aus("NV",$a);}
function LD($a="") {return deflos_aus("LD",$a);}

function AT_($a="") {return deflos_tl("AT",$a);}
function AV_($a="") {return deflos_tl("AV",$a);}
function NT_($a="") {return deflos_tl("NT",$a);}
function NV_($a="") {return deflos_tl("NV",$a);}
function LD_($a="") {return deflos_tl("LD",$a);}

